# flake8: noqa

import pprint
import subprocess
from os.path import join, basename, dirname

import click

from rastervision.pipeline.file_system import (
    list_paths, download_or_copy, file_exists, make_dir, file_to_json)

cfg = [
    {
        'key':
        'spacenet-rio-cc',
        'module':
        'rastervision.pytorch_backend.examples.chip_classification.spacenet_rio',
        'local': {
            'raw_uri': '/opt/data/raw-data/spacenet-dataset',
            'processed_uri': '/opt/data/examples/spacenet/rio/processed-data',
            'root_uri': '/opt/data/examples/spacenet-rio-cc'
        },
        'remote': {
            'raw_uri':
            's3://spacenet-dataset/',
            'processed_uri':
            's3://raster-vision-lf-dev/examples/spacenet/rio/processed-data',
            'root_uri':
            's3://raster-vision-lf-dev/examples/spacenet-rio-cc'
        },
        'sample_img':
        'https://s3.amazonaws.com/azavea-research-public-data/raster-vision/examples/model-zoo-0.12/spacenet-rio-cc/013022223130_sample.tif'
    },
    {
        'key':
        'isprs-potsdam-ss',
        'module':
        'rastervision.pytorch_backend.examples.semantic_segmentation.isprs_potsdam',
        'local': {
            'raw_uri': '/opt/data/raw-data/isprs-potsdam/',
            'processed_uri': '/opt/data/examples/potsdam/processed-data',
            'root_uri': '/opt/data/examples/isprs-potsdam-ss/'
        },
        'remote': {
            'raw_uri':
            's3://raster-vision-raw-data/isprs-potsdam',
            'processed_uri':
            's3://raster-vision-lf-dev/examples/potsdam/processed-data',
            'root_uri':
            's3://raster-vision-lf-dev/examples/isprs-potsdam-ss'
        },
        'sample_img':
        'https://s3.amazonaws.com/azavea-research-public-data/raster-vision/examples/model-zoo-0.12/isprs-potsdam-ss/3_12_sample.tif'
    },
    {
        'key':
        'spacenet-vegas-buildings-ss',
        'module':
        'rastervision.pytorch_backend.examples.semantic_segmentation.spacenet_vegas',
        'local': {
            'raw_uri': 's3://spacenet-dataset/',
            'root_uri': '/opt/data/examples/spacenet-vegas-buildings-ss'
        },
        'remote': {
            'raw_uri':
            's3://spacenet-dataset/',
            'root_uri':
            's3://raster-vision-lf-dev/examples/spacenet-vegas-buildings-ss'
        },
        'extra_args': [['target', 'buildings']],
        'sample_img':
        'https://s3.amazonaws.com/azavea-research-public-data/raster-vision/examples/model-zoo-0.12/spacenet-vegas-buildings-ss/1929.tif'
    },
    {
        'key':
        'spacenet-vegas-roads-ss',
        'module':
        'rastervision.pytorch_backend.examples.semantic_segmentation.spacenet_vegas',
        'local': {
            'raw_uri': 's3://spacenet-dataset/',
            'root_uri': '/opt/data/examples/spacenet-vegas-roads-ss'
        },
        'remote': {
            'raw_uri':
            's3://spacenet-dataset/',
            'root_uri':
            's3://raster-vision-lf-dev/examples/spacenet-vegas-roads-ss'
        },
        'extra_args': [['target', 'roads']],
        'sample_img':
        'https://s3.amazonaws.com/azavea-research-public-data/raster-vision/examples/model-zoo-0.12/spacenet-vegas-roads-ss/524.tif'
    },
    {
        'key':
        'cowc-potsdam-od',
        'module':
        'rastervision.pytorch_backend.examples.object_detection.cowc_potsdam',
        'local': {
            'raw_uri': '/opt/data/raw-data/isprs-potsdam',
            'processed_uri': '/opt/data/examples/cowc-potsdam/processed-data',
            'root_uri': '/opt/data/examples/cowc-potsdam-od'
        },
        'remote': {
            'raw_uri':
            's3://raster-vision-raw-data/isprs-potsdam',
            'processed_uri':
            's3://raster-vision-lf-dev/examples/cowc-potsdam/processed-data',
            'root_uri':
            's3://raster-vision-lf-dev/examples/cowc-potsdam-od'
        },
        'sample_img':
        'https://s3.amazonaws.com/azavea-research-public-data/raster-vision/examples/model-zoo-0.12/cowc-potsdam-od/3_10_sample.tif'
    },
    {
        'key':
        'xview-od',
        'module':
        'rastervision.pytorch_backend.examples.object_detection.xview',
        'local': {
            'raw_uri': 's3://raster-vision-xview-example/raw-data',
            'processed_uri': '/opt/data/examples/xview/processed-data',
            'root_uri': '/opt/data/examples/xview-od'
        },
        'remote': {
            'raw_uri':
            's3://raster-vision-xview-example/raw-data',
            'processed_uri':
            's3://raster-vision-lf-dev/examples/xview/processed-data',
            'root_uri':
            's3://raster-vision-lf-dev/examples/xview-od'
        },
        'sample_img':
        'https://s3.amazonaws.com/azavea-research-public-data/raster-vision/examples/model-zoo-0.12/xview-od/1124-sample.tif'
    },
]


def validate_keys(keys):
    exp_keys = [exp_cfg['key'] for exp_cfg in cfg]
    invalid_keys = set(keys).difference(exp_keys)
    if invalid_keys:
        raise ValueError('{} are invalid keys'.format(', '.join(invalid_keys)))


def _run(exp_cfg, output_dir, test=True, remote=False, commands=None):
    uris = exp_cfg['remote'] if remote else exp_cfg['local']
    cmd = ['rastervision']
    rv_profile = exp_cfg.get('rv_profile')
    if rv_profile is not None:
        cmd += ['-p', rv_profile]
    cmd += ['run', 'batch' if remote else 'inprocess', exp_cfg['module']]
    if commands is not None:
        cmd += commands
    cmd += ['-a', 'raw_uri', uris['raw_uri']]
    if 'processed_uri' in uris:
        cmd += ['-a', 'processed_uri', uris['processed_uri']]
    root_uri = join(uris['root_uri'], output_dir)
    cmd += ['-a', 'root_uri', root_uri]
    cmd += ['-a', 'test', 'True' if test else 'False']
    extra_args = exp_cfg.get('extra_args')
    if extra_args:
        for k, v in extra_args:
            cmd += ['-a', str(k), str(v)]
    if remote:
        cmd += ['--splits', '3']

    print('running command:')
    print(' '.join(cmd))
    proc = subprocess.run(cmd)
    if proc.returncode != 0:
        print('failure!')
        print(' '.join(cmd))
        exit()


@click.group()
def test():
    pass


@test.command()
@click.argument('output_dir')
@click.argument('keys', nargs=-1)
@click.option('--test', is_flag=True)
@click.option('--remote', is_flag=True)
@click.option('--commands')
def run(output_dir, keys, test, remote, commands):
    run_all = len(keys) == 0
    validate_keys(keys)

    if commands is not None:
        commands = commands.split(' ')
    for exp_cfg in cfg:
        if run_all or exp_cfg['key'] in keys:
            _run(
                exp_cfg,
                output_dir,
                test=test,
                remote=remote,
                commands=commands)


def _collect(key, root_uri, output_dir, collect_dir, get_model_bundle=False):
    print('\nCollecting experiment {}...\n'.format(key))

    model_bundle_uri = join(root_uri, output_dir, 'bundle', 'model-bundle.zip')
    eval_uri = join(root_uri, output_dir, 'eval', 'eval.json')

    if not file_exists(eval_uri):
        print('Missing eval!')
        return

    if not file_exists(model_bundle_uri):
        print('Missing model bundle!')
        return

    make_dir(join(collect_dir, key))
    if get_model_bundle:
        download_or_copy(model_bundle_uri, join(collect_dir, key))

    download_or_copy(eval_uri, join(collect_dir, key))

    eval_json = file_to_json(join(collect_dir, key, 'eval.json'))
    pprint.pprint(eval_json['overall'], indent=4)


@test.command()
@click.argument('output_dir')
@click.argument('collect_dir')
@click.argument('keys', nargs=-1)
@click.option('--remote', is_flag=True)
@click.option('--get-model-bundle', is_flag=True)
def collect(output_dir, collect_dir, keys, remote, get_model_bundle):
    run_all = len(keys) == 0
    validate_keys(keys)

    for exp_cfg in cfg:
        if run_all or exp_cfg['key'] in keys:
            key = exp_cfg['key']
            uris = exp_cfg['remote'] if remote else exp_cfg['local']
            root_uri = uris['root_uri']
            _collect(key, root_uri, output_dir, collect_dir, get_model_bundle)


def _predict(key, sample_img, collect_dir):
    print('\nTesting model bundle for {}...\n'.format(key))

    model_bundle_uri = join(collect_dir, key, 'model-bundle.zip')
    if not file_exists(model_bundle_uri):
        print('Bundle does not exist!')
        return

    sample_img = download_or_copy(sample_img, join(collect_dir, key))

    exts = {'cc': 'json', 'ss': 'tif', 'od': 'json'}
    for task, ext in exts.items():
        if key.endswith(task):
            break

    out_uri = join(collect_dir, key, 'predictions.{}'.format(ext))
    cmd = ['rastervision', 'predict', model_bundle_uri, sample_img, out_uri]
    proc = subprocess.run(cmd)
    if proc.returncode != 0:
        print('failure!')
        print(' '.join(cmd))
        exit()


@test.command()
@click.argument('collect_dir')
@click.argument('keys', nargs=-1)
def predict(collect_dir, keys):
    run_all = len(keys) == 0
    validate_keys(keys)

    for exp_cfg in cfg:
        if run_all or exp_cfg['key'] in keys:
            key = exp_cfg['key']
            _predict(key, exp_cfg['sample_img'], collect_dir)


if __name__ == '__main__':
    test()



from os.path import join
import uuid

import numpy as np

from rastervision.pipeline.file_system import (make_dir)
from rastervision.core.data.label import SemanticSegmentationLabels
from rastervision.core.utils.misc import save_img
from rastervision.core.data_sample import DataSample
from rastervision.pytorch_backend.pytorch_learner_backend import (
    PyTorchLearnerSampleWriter, PyTorchLearnerBackend)


class PyTorchSemanticSegmentationSampleWriter(PyTorchLearnerSampleWriter):
    def write_sample(self, sample: DataSample):
        """
        This writes a training or validation sample to
        (train|valid)/img/{scene_id}-{ind}.png and
        (train|valid)/labels/{scene_id}-{ind}.png
        """
        split_name = 'train' if sample.is_train else 'valid'
        label_arr = sample.labels.get_label_arr(sample.window).astype(np.uint8)

        img_dir = join(self.sample_dir, split_name, 'img')
        labels_dir = join(self.sample_dir, split_name, 'labels')
        make_dir(img_dir)
        make_dir(labels_dir)

        img_path = join(img_dir, '{}-{}.png'.format(sample.scene_id,
                                                    self.sample_ind))
        labels_path = join(
            labels_dir, '{}-{}.png'.format(sample.scene_id, self.sample_ind))
        save_img(sample.chip, img_path)
        save_img(label_arr, labels_path)

        self.sample_ind += 1


class PyTorchSemanticSegmentation(PyTorchLearnerBackend):
    def get_sample_writer(self):
        output_uri = join(self.pipeline_cfg.chip_uri, '{}.zip'.format(
            str(uuid.uuid4())))
        return PyTorchSemanticSegmentationSampleWriter(
            output_uri, self.pipeline_cfg.dataset.class_config, self.tmp_dir)

    def predict(self, chips, windows):
        if self.learner is None:
            self.load_model()

        batch_out = self.learner.numpy_predict(chips, raw_out=False)
        labels = SemanticSegmentationLabels()
        for out, window in zip(batch_out, windows):
            labels.set_label_arr(window, out)

        return labels



import os
import glob
import subprocess

import click

from rv.utils import make_empty_dir


@click.command()
@click.argument('input_dir')
@click.argument('output_dir')
@click.option(
    '--resolution', default=0.3, help='Output resolution in meters/pixel')
def resample_geotiffs(input_dir, output_dir, resolution):
    input_paths = glob.glob(os.path.join(input_dir, '*.tif'))
    make_empty_dir(output_dir)

    for input_path in input_paths:
        input_filename = os.path.basename(input_path)
        subprocess.call([
            'gdalwarp', '-tr',
            str(resolution),
            str(-resolution), '-r', 'bilinear', '-setci', '-co', 'ALPHA=NO',
            input_path,
            os.path.join(output_dir, input_filename)
        ])


if __name__ == '__main__':
    resample_geotiffs()



# flake8: noqa

import os
from os.path import join

from rastervision.core.rv_pipeline import *
from rastervision.core.backend import *
from rastervision.core.data import *
from rastervision.core.analyzer import *
from rastervision.pytorch_backend import *
from rastervision.pytorch_learner import *
from rastervision.pytorch_backend.examples.utils import get_scene_info, save_image_crop


def get_config(runner, raw_uri, processed_uri, root_uri, test=False):
    train_ids = [
        '2_10', '2_11', '2_12', '2_14', '3_11', '3_13', '4_10', '5_10', '6_7',
        '6_9'
    ]
    val_ids = ['2_13', '6_8', '3_10']

    if test:
        train_ids = train_ids[0:1]
        val_ids = val_ids[0:1]

    def make_scene(id):
        raster_uri = join(raw_uri,
                          '4_Ortho_RGBIR/top_potsdam_{}_RGBIR.tif'.format(id))
        label_uri = join(processed_uri, 'labels', 'all',
                         'top_potsdam_{}_RGBIR.json'.format(id))

        if test:
            crop_uri = join(processed_uri, 'crops',
                            os.path.basename(raster_uri))
            save_image_crop(
                raster_uri,
                crop_uri,
                label_uri=label_uri,
                size=1000,
                min_features=5)
            raster_uri = crop_uri

        raster_source = RasterioSourceConfig(
            uris=[raster_uri], channel_order=[0, 1, 2])

        label_source = ObjectDetectionLabelSourceConfig(
            vector_source=GeoJSONVectorSourceConfig(
                uri=label_uri, default_class_id=0, ignore_crs_field=True))

        return SceneConfig(
            id=id, raster_source=raster_source, label_source=label_source)

    class_config = ClassConfig(names=['vehicle'])
    chip_sz = 300
    dataset = DatasetConfig(
        class_config=class_config,
        train_scenes=[make_scene(id) for id in train_ids],
        validation_scenes=[make_scene(id) for id in val_ids])
    chip_options = ObjectDetectionChipOptions(neg_ratio=5.0, ioa_thresh=0.9)
    predict_options = ObjectDetectionPredictOptions(
        merge_thresh=0.5, score_thresh=0.9)

    backend = PyTorchObjectDetectionConfig(
        model=ObjectDetectionModelConfig(backbone=Backbone.resnet18),
        solver=SolverConfig(
            lr=1e-4,
            num_epochs=10,
            test_num_epochs=2,
            batch_sz=16,
            one_cycle=True),
        log_tensorboard=True,
        run_tensorboard=False,
        test_mode=test)

    return ObjectDetectionConfig(
        root_uri=root_uri,
        dataset=dataset,
        backend=backend,
        train_chip_sz=chip_sz,
        predict_chip_sz=chip_sz,
        chip_options=chip_options,
        predict_options=predict_options)



from typing import List

from rastervision.pipeline.config import register_config, Field
from rastervision.core.backend import BackendConfig
from rastervision.pytorch_learner.learner_config import (
    SolverConfig, ModelConfig, default_augmentors, augmentors as
    augmentor_list)


@register_config('pytorch_learner_backend')
class PyTorchLearnerBackendConfig(BackendConfig):
    model: ModelConfig
    solver: SolverConfig
    log_tensorboard: bool = Field(
        True, description='If True, log events to Tensorboard log files.')
    run_tensorboard: bool = Field(
        False,
        description='If True, run Tensorboard server pointing at log files.')
    augmentors: List[str] = Field(
        default_augmentors,
        description=(
            'Names of albumentations augmentors to use for training batches. '
            'Choices include: ' + str(augmentor_list)))
    test_mode: bool = Field(
        False,
        description=
        ('This field is passed along to the LearnerConfig which is returned by '
         'get_learner_config(). For more info, see the docs for'
         'pytorch_learner.learner_config.LearnerConfig.test_mode.'))

    def get_bundle_filenames(self):
        return ['model-bundle.zip']

    def get_learner_config(self, pipeline):
        raise NotImplementedError()

    def build(self, pipeline, tmp_dir):
        raise NotImplementedError()



from rastervision.pipeline.config import register_config
from rastervision.pytorch_backend.pytorch_learner_backend_config import (
    PyTorchLearnerBackendConfig)
from rastervision.pytorch_learner.semantic_segmentation_learner_config import (
    SemanticSegmentationModelConfig, SemanticSegmentationLearnerConfig,
    SemanticSegmentationDataConfig)
from rastervision.pytorch_backend.pytorch_semantic_segmentation import (
    PyTorchSemanticSegmentation)


@register_config('pytorch_semantic_segmentation_backend')
class PyTorchSemanticSegmentationConfig(PyTorchLearnerBackendConfig):
    model: SemanticSegmentationModelConfig

    def get_learner_config(self, pipeline):
        data = SemanticSegmentationDataConfig()
        data.uri = pipeline.chip_uri
        data.class_names = pipeline.dataset.class_config.names
        data.class_colors = pipeline.dataset.class_config.colors
        data.img_sz = pipeline.train_chip_sz
        data.augmentors = self.augmentors

        learner = SemanticSegmentationLearnerConfig(
            data=data,
            model=self.model,
            solver=self.solver,
            test_mode=self.test_mode,
            output_uri=pipeline.train_uri,
            log_tensorboard=self.log_tensorboard,
            run_tensorboard=self.run_tensorboard)
        learner.update()
        return learner

    def build(self, pipeline, tmp_dir):
        learner = self.get_learner_config(pipeline)
        return PyTorchSemanticSegmentation(pipeline, learner, tmp_dir)



#!/usr/bin/env python

import os
import re
import sys

from subprocess import check_output, call

if not len(sys.argv) >= 4:
    print('Usage: {} <input_rgb.tif> <input_label.tif> <output_label.tif>'.
          format(sys.argv[0]))
    exit()

input_rgb = sys.argv[1]
input_label = sys.argv[2]
output_label = sys.argv[3]

# Get proj4 string
proj4 = check_output(['gdalsrsinfo', '-o', 'proj4', input_rgb], stderr=None)
proj4 = proj4[1:-2]

# Get upper left, lower right info
with open(os.devnull, 'w') as devnull:
    ullr = check_output(
        ['gdalinfo', input_rgb], stderr=devnull).decode('utf-8')
ul_re = re.compile(r'^Upper Left.*?([0-9\.]+).*?([0-9\.]+)', re.MULTILINE)
lr_re = re.compile(r'^Lower Right.*?([0-9\.]+).*?([0-9\.]+)', re.MULTILINE)
ul = re.search(ul_re, ullr)
lr = re.search(lr_re, ullr)

args = [
    'gdal_translate', '-a_srs', proj4, '-a_ullr',
    ul.group(1),
    ul.group(2),
    lr.group(1),
    lr.group(2), input_label, output_label
]

call(args)



# flake8: noqa

from os.path import join

from rastervision.core.rv_pipeline import *
from rastervision.core.backend import *
from rastervision.core.data import *
from rastervision.pytorch_backend import *
from rastervision.pytorch_learner import *


def get_config(runner):
    root_uri = '/opt/data/output/'
    base_uri = ('https://s3.amazonaws.com/azavea-research-public-data/'
                'raster-vision/examples/spacenet')
    train_image_uri = '{}/RGB-PanSharpen_AOI_2_Vegas_img205.tif'.format(
        base_uri)
    train_label_uri = '{}/buildings_AOI_2_Vegas_img205.geojson'.format(
        base_uri)
    val_image_uri = '{}/RGB-PanSharpen_AOI_2_Vegas_img25.tif'.format(base_uri)
    val_label_uri = '{}/buildings_AOI_2_Vegas_img25.geojson'.format(base_uri)
    channel_order = [0, 1, 2]
    class_config = ClassConfig(
        names=['building', 'background'], colors=['red', 'black'])

    def make_scene(scene_id, image_uri, label_uri):
        """
        - StatsTransformer is used to convert uint16 values to uint8.
        - The GeoJSON does not have a class_id property for each geom,
          so it is inferred as 0 (ie. building) because the default_class_id
          is set to 0.
        - The labels are in the form of GeoJSON which needs to be rasterized
          to use as label for semantic segmentation, so we use a RasterizedSource.
        - The rasterizer set the background (as opposed to foreground) pixels
          to 1 because background_class_id is set to 1.
        """
        raster_source = RasterioSourceConfig(
            uris=[image_uri],
            channel_order=channel_order,
            transformers=[StatsTransformerConfig()])
        vector_source = GeoJSONVectorSourceConfig(
            uri=label_uri, default_class_id=0, ignore_crs_field=True)
        label_source = SemanticSegmentationLabelSourceConfig(
            raster_source=RasterizedSourceConfig(
                vector_source=vector_source,
                rasterizer_config=RasterizerConfig(background_class_id=1)))
        return SceneConfig(
            id=scene_id,
            raster_source=raster_source,
            label_source=label_source)

    dataset = DatasetConfig(
        class_config=class_config,
        train_scenes=[
            make_scene('scene_205', train_image_uri, train_label_uri)
        ],
        validation_scenes=[
            make_scene('scene_25', val_image_uri, val_label_uri)
        ])

    # Use the PyTorch backend for the SemanticSegmentation pipeline.
    chip_sz = 300
    backend = PyTorchSemanticSegmentationConfig(
        model=SemanticSegmentationModelConfig(backbone=Backbone.resnet50),
        solver=SolverConfig(lr=1e-4, num_epochs=1, batch_sz=2))
    chip_options = SemanticSegmentationChipOptions(
        window_method=SemanticSegmentationWindowMethod.random_sample,
        chips_per_scene=10)

    return SemanticSegmentationConfig(
        root_uri=root_uri,
        dataset=dataset,
        backend=backend,
        train_chip_sz=chip_sz,
        predict_chip_sz=chip_sz,
        chip_options=chip_options)



# flake8: noqa

import os
from os.path import join

from rastervision.core.rv_pipeline import *
from rastervision.core.backend import *
from rastervision.core.data import *
from rastervision.core.analyzer import *
from rastervision.pytorch_backend import *
from rastervision.pytorch_learner import *
from rastervision.pytorch_backend.examples.utils import get_scene_info, save_image_crop


def get_config(runner, raw_uri, processed_uri, root_uri, test=False):
    train_ids = [
        '2-10', '2-11', '3-10', '3-11', '4-10', '4-11', '4-12', '5-10', '5-11',
        '5-12', '6-10', '6-11', '6-7', '6-9', '7-10', '7-11', '7-12', '7-7',
        '7-8', '7-9'
    ]
    val_ids = ['2-12', '3-12', '6-12']

    if test:
        train_ids = train_ids[0:1]
        val_ids = val_ids[0:1]

    class_config = ClassConfig(
        names=[
            'Car', 'Building', 'Low Vegetation', 'Tree', 'Impervious',
            'Clutter'
        ],
        colors=[
            '#ffff00', '#0000ff', '#00ffff', '#00ff00', '#ffffff', '#ff0000'
        ])

    def make_scene(id):
        id = id.replace('-', '_')
        raster_uri = '{}/4_Ortho_RGBIR/top_potsdam_{}_RGBIR.tif'.format(
            raw_uri, id)
        label_uri = '{}/5_Labels_for_participants/top_potsdam_{}_label.tif'.format(
            raw_uri, id)

        if test:
            crop_uri = join(processed_uri, 'crops',
                            os.path.basename(raster_uri))
            label_crop_uri = join(processed_uri, 'crops',
                                  os.path.basename(label_uri))
            save_image_crop(
                raster_uri,
                crop_uri,
                label_uri=label_uri,
                label_crop_uri=label_crop_uri,
                size=600,
                vector_labels=False)
            raster_uri = crop_uri
            label_uri = label_crop_uri

        # infrared, red, green
        channel_order = [3, 0, 1]
        raster_source = RasterioSourceConfig(
            uris=[raster_uri], channel_order=channel_order)

        # Using with_rgb_class_map because label TIFFs have classes encoded as
        # RGB colors.
        label_source = SemanticSegmentationLabelSourceConfig(
            rgb_class_config=class_config,
            raster_source=RasterioSourceConfig(uris=[label_uri]))

        # URI will be injected by scene config.
        # Using rgb=True because we want prediction TIFFs to be in
        # RGB format.
        label_store = SemanticSegmentationLabelStoreConfig(
            rgb=True, vector_output=[PolygonVectorOutputConfig(class_id=0)])

        scene = SceneConfig(
            id=id,
            raster_source=raster_source,
            label_source=label_source,
            label_store=label_store)

        return scene

    dataset = DatasetConfig(
        class_config=class_config,
        train_scenes=[make_scene(id) for id in train_ids],
        validation_scenes=[make_scene(id) for id in val_ids])
    chip_sz = 300
    chip_options = SemanticSegmentationChipOptions(
        window_method=SemanticSegmentationWindowMethod.sliding, stride=chip_sz)

    backend = PyTorchSemanticSegmentationConfig(
        model=SemanticSegmentationModelConfig(backbone=Backbone.resnet50),
        solver=SolverConfig(
            lr=1e-4,
            num_epochs=10,
            test_num_epochs=2,
            batch_sz=8,
            one_cycle=True),
        log_tensorboard=True,
        run_tensorboard=False,
        test_mode=test)

    return SemanticSegmentationConfig(
        root_uri=root_uri,
        dataset=dataset,
        backend=backend,
        train_chip_sz=chip_sz,
        predict_chip_sz=chip_sz,
        chip_options=chip_options)



import os
import glob

import rasterio
from PIL import Image
import numpy as np
import click

from object_detection.utils.np_box_list import BoxList

from rv.utils import save_geojson, make_empty_dir


def png_to_geojson(geotiff_path, label_png_path, output_path, object_half_len):
    """Convert COWC PNG labels to GeoJSON format.
    In the COWC dataset, the center position of cars is represented as
    non-zero pixels in PNG files that are aligned with the GeoTIFFs.
    This script converts the PNG file to a GeoJSON representation.
    """
    image_dataset = rasterio.open(geotiff_path)
    label_im = np.array(Image.open(label_png_path))

    point_inds = np.argwhere(label_im[:, :, 0] != 0).astype(np.float)

    # Normalize inds
    point_inds[:, 0] /= label_im.shape[0]
    point_inds[:, 1] /= label_im.shape[1]

    # Convert to geotiff image inds
    point_inds[:, 0] *= image_dataset.height
    point_inds[:, 1] *= image_dataset.width
    point_inds = point_inds.astype(np.int)

    # Turn points into squares and ensure edges aren't outside the array
    y_min = np.clip(point_inds[:, 0:1] - object_half_len, 0,
                    image_dataset.height)
    x_min = np.clip(point_inds[:, 1:2] - object_half_len, 0,
                    image_dataset.width)
    y_max = np.clip(point_inds[:, 0:1] + object_half_len, 0,
                    image_dataset.height)
    x_max = np.clip(point_inds[:, 1:2] + object_half_len, 0,
                    image_dataset.width)

    # Write to GeoJSON
    boxes = np.hstack([y_min, x_min, y_max, x_max]).astype(np.float)
    boxlist = BoxList(boxes)
    save_geojson(output_path, boxlist, image_dataset=image_dataset)
    return boxlist


@click.command()
@click.argument('geotiff_dir')
@click.argument('label_png_dir')
@click.argument('output_dir')
@click.option('--object-half-len', default=50)
def prepare_potsdam(geotiff_dir, label_png_dir, output_dir, object_half_len):
    label_paths = glob.glob(
        os.path.join(label_png_dir, 'top_potsdam_*_RGB_Annotated_Cars.png'))
    make_empty_dir(output_dir)

    for label_path in label_paths:
        geotiff_base = os.path.basename(label_path)[0:-19]
        geotiff_path = os.path.join(geotiff_dir, geotiff_base + 'IR.tif')
        output_path = os.path.join(output_dir, geotiff_base + 'IR.json')
        boxlist = png_to_geojson(
            geotiff_path,
            label_path,
            output_path,
            object_half_len=object_half_len)
        print('Saved {} with {} boxes.'.format(output_path,
                                               boxlist.num_boxes()))


if __name__ == '__main__':
    prepare_potsdam()



from os.path import join
import uuid

from rastervision.pipeline.file_system import (make_dir, json_to_file)
from rastervision.core.data.label import ObjectDetectionLabels
from rastervision.core.utils.misc import save_img
from rastervision.core.data_sample import DataSample
from rastervision.pytorch_backend.pytorch_learner_backend import (
    PyTorchLearnerSampleWriter, PyTorchLearnerBackend)


class PyTorchObjectDetectionSampleWriter(PyTorchLearnerSampleWriter):
    """Writes data in COCO format."""

    def __enter__(self):
        super().__enter__()

        self.splits = {
            'train': {
                'images': [],
                'annotations': []
            },
            'valid': {
                'images': [],
                'annotations': []
            }
        }
        self.categories = [{
            'id': class_id,
            'name': class_name
        } for class_id, class_name in enumerate(self.class_config.names)]

        return self

    def __exit__(self, type, value, traceback):
        """This writes label files in COCO format to (train|valid)/labels.json"""
        for split in ['train', 'valid']:
            if len(self.splits[split]['images']) > 0:
                split_dir = join(self.sample_dir, split)
                labels_path = join(split_dir, 'labels.json')

                images = self.splits[split]['images']
                annotations = self.splits[split]['annotations']
                coco_dict = {
                    'images': images,
                    'annotations': annotations,
                    'categories': self.categories
                }
                json_to_file(coco_dict, labels_path)

        super().__exit__(type, value, traceback)

    def write_sample(self, sample: DataSample):
        """
        This writes a training or validation sample to
        (train|valid)/img/{scene_id}-{ind}.png and updates
        some COCO data structures.
        """
        split = 'train' if sample.is_train else 'valid'
        split_dir = join(self.sample_dir, split)
        img_dir = join(split_dir, 'img')
        make_dir(img_dir)
        img_fn = '{}-{}.png'.format(sample.scene_id, self.sample_ind)
        img_path = join(img_dir, img_fn)
        save_img(sample.chip, img_path)

        images = self.splits[split]['images']
        annotations = self.splits[split]['annotations']

        images.append({
            'file_name': img_fn,
            'id': self.sample_ind,
            'height': sample.chip.shape[0],
            'width': sample.chip.shape[1]
        })

        npboxes = sample.labels.get_npboxes()
        npboxes = ObjectDetectionLabels.global_to_local(npboxes, sample.window)
        for box_ind, (box, class_id) in enumerate(
                zip(npboxes, sample.labels.get_class_ids())):
            bbox = [box[1], box[0], box[3] - box[1], box[2] - box[0]]
            bbox = [int(i) for i in bbox]
            annotations.append({
                'id': '{}-{}'.format(self.sample_ind, box_ind),
                'image_id': self.sample_ind,
                'bbox': bbox,
                'category_id': int(class_id)
            })

        self.sample_ind += 1


class PyTorchObjectDetection(PyTorchLearnerBackend):
    def get_sample_writer(self):
        output_uri = join(self.pipeline_cfg.chip_uri, '{}.zip'.format(
            str(uuid.uuid4())))
        return PyTorchObjectDetectionSampleWriter(
            output_uri, self.pipeline_cfg.dataset.class_config, self.tmp_dir)

    def predict(self, chips, windows):
        """Return predictions for a chip using model.

        Args:
            chips: [[height, width, channels], ...] numpy array of chips
            windows: List of boxes that are the windows aligned with the chips.

        Return:
            Labels object containing predictions
        """
        if self.learner is None:
            self.load_model()

        batch_out = self.learner.numpy_predict(chips, raw_out=False)
        labels = ObjectDetectionLabels.make_empty()

        for chip_ind, out in enumerate(batch_out):
            window = windows[chip_ind]
            boxes = out['boxes']
            class_ids = out['class_ids']
            scores = out['scores']
            boxes = ObjectDetectionLabels.local_to_global(boxes, window)
            labels += ObjectDetectionLabels(boxes, class_ids, scores=scores)

        return labels



# flake8: noqa

import os
from os.path import join

from rastervision.core.rv_pipeline import *
from rastervision.core.backend import *
from rastervision.core.data import *
from rastervision.core.analyzer import *
from rastervision.pytorch_backend import *
from rastervision.pytorch_learner import *
from rastervision.pytorch_backend.examples.utils import get_scene_info, save_image_crop

aoi_path = 'AOIs/AOI_1_Rio/srcData/buildingLabels/Rio_OUTLINE_Public_AOI.geojson'


def get_config(runner, raw_uri, processed_uri, root_uri, test=False):
    debug = False
    train_scene_info = get_scene_info(join(processed_uri, 'train-scenes.csv'))
    val_scene_info = get_scene_info(join(processed_uri, 'val-scenes.csv'))
    log_tensorboard = True
    run_tensorboard = True
    class_config = ClassConfig(names=['no_building', 'building'])

    if test:
        debug = True
        train_scene_info = train_scene_info[0:1]
        val_scene_info = val_scene_info[0:1]

    def make_scene(scene_info):
        (raster_uri, label_uri) = scene_info
        raster_uri = join(raw_uri, raster_uri)
        label_uri = join(processed_uri, label_uri)
        aoi_uri = join(raw_uri, aoi_path)

        if test:
            crop_uri = join(processed_uri, 'crops',
                            os.path.basename(raster_uri))
            label_crop_uri = join(processed_uri, 'crops',
                                  os.path.basename(label_uri))

            save_image_crop(
                raster_uri,
                crop_uri,
                label_uri=label_uri,
                label_crop_uri=label_crop_uri,
                size=600,
                min_features=20,
                class_config=class_config)
            raster_uri = crop_uri
            label_uri = label_crop_uri

        id = os.path.splitext(os.path.basename(raster_uri))[0]
        raster_source = RasterioSourceConfig(
            channel_order=[0, 1, 2], uris=[raster_uri])
        label_source = ChipClassificationLabelSourceConfig(
            vector_source=GeoJSONVectorSourceConfig(
                uri=label_uri, default_class_id=1, ignore_crs_field=True),
            ioa_thresh=0.5,
            use_intersection_over_cell=False,
            pick_min_class_id=False,
            background_class_id=0,
            infer_cells=True)

        return SceneConfig(
            id=id,
            raster_source=raster_source,
            label_source=label_source,
            aoi_uris=[aoi_uri])

    chip_sz = 200
    train_scenes = [make_scene(info) for info in train_scene_info]
    val_scenes = [make_scene(info) for info in val_scene_info]
    dataset = DatasetConfig(
        class_config=class_config,
        train_scenes=train_scenes,
        validation_scenes=val_scenes)

    model = ClassificationModelConfig(backbone=Backbone.resnet50)
    solver = SolverConfig(
        lr=1e-4, num_epochs=20, test_num_epochs=3, batch_sz=32, one_cycle=True)
    backend = PyTorchChipClassificationConfig(
        model=model,
        solver=solver,
        log_tensorboard=log_tensorboard,
        run_tensorboard=run_tensorboard,
        test_mode=test)

    config = ChipClassificationConfig(
        root_uri=root_uri,
        dataset=dataset,
        backend=backend,
        train_chip_sz=chip_sz,
        predict_chip_sz=chip_sz)
    return config



# flake8: noqa

import rastervision.pipeline
from rastervision.pytorch_backend.pytorch_chip_classification_config import *
from rastervision.pytorch_backend.pytorch_chip_classification import *
from rastervision.pytorch_backend.pytorch_semantic_segmentation_config import *
from rastervision.pytorch_backend.pytorch_semantic_segmentation import *
from rastervision.pytorch_backend.pytorch_object_detection_config import *
from rastervision.pytorch_backend.pytorch_object_detection import *


def register_plugin(registry):
    registry.set_plugin_version('rastervision.pytorch_backend', 0)
    registry.set_plugin_aliases('rastervision.pytorch_backend',
                                ['rastervision2.pytorch_backend'])



from rastervision.pipeline.config import register_config
from rastervision.pytorch_backend.pytorch_learner_backend_config import (
    PyTorchLearnerBackendConfig)
from rastervision.pytorch_learner.classification_learner_config import (
    ClassificationModelConfig, ClassificationLearnerConfig,
    ClassificationDataConfig)

from rastervision.pytorch_backend.pytorch_chip_classification import (
    PyTorchChipClassification)


@register_config('pytorch_chip_classification_backend')
class PyTorchChipClassificationConfig(PyTorchLearnerBackendConfig):
    model: ClassificationModelConfig

    def get_learner_config(self, pipeline):
        data = ClassificationDataConfig()
        data.uri = pipeline.chip_uri
        data.class_names = pipeline.dataset.class_config.names
        data.class_colors = pipeline.dataset.class_config.colors
        data.img_sz = pipeline.train_chip_sz
        data.augmentors = self.augmentors

        learner = ClassificationLearnerConfig(
            data=data,
            model=self.model,
            solver=self.solver,
            test_mode=self.test_mode,
            output_uri=pipeline.train_uri,
            log_tensorboard=self.log_tensorboard,
            run_tensorboard=self.run_tensorboard)
        learner.update()
        return learner

    def build(self, pipeline, tmp_dir):
        learner = self.get_learner_config(pipeline)
        return PyTorchChipClassification(pipeline, learner, tmp_dir)



# flake8: noqa

import os
from os.path import join

from rastervision.core.rv_pipeline import *
from rastervision.core.backend import *
from rastervision.core.data import *
from rastervision.core.analyzer import *
from rastervision.pytorch_backend import *
from rastervision.pytorch_learner import *
from rastervision.pytorch_backend.examples.utils import get_scene_info, save_image_crop


def get_config(runner, raw_uri, processed_uri, root_uri, test=False):
    train_scene_info = get_scene_info(join(processed_uri, 'train-scenes.csv'))
    val_scene_info = get_scene_info(join(processed_uri, 'val-scenes.csv'))
    if test:
        train_scene_info = train_scene_info[0:1]
        val_scene_info = val_scene_info[0:1]

    def make_scene(scene_info):
        (raster_uri, label_uri) = scene_info
        raster_uri = join(raw_uri, raster_uri)
        label_uri = join(processed_uri, label_uri)

        if test:
            crop_uri = join(processed_uri, 'crops',
                            os.path.basename(raster_uri))
            save_image_crop(raster_uri, crop_uri, size=600, min_features=5)
            raster_uri = crop_uri

        id = os.path.splitext(os.path.basename(raster_uri))[0]

        raster_source = RasterioSourceConfig(
            uris=[raster_uri], channel_order=[0, 1, 2])

        label_source = ObjectDetectionLabelSourceConfig(
            vector_source=GeoJSONVectorSourceConfig(
                uri=label_uri, default_class_id=0, ignore_crs_field=True))

        return SceneConfig(
            id=id, raster_source=raster_source, label_source=label_source)

    train_scenes = [make_scene(info) for info in train_scene_info]
    val_scenes = [make_scene(info) for info in val_scene_info]
    class_config = ClassConfig(names=['vehicle'], colors=['red'])
    chip_sz = 300
    dataset = DatasetConfig(
        class_config=class_config,
        train_scenes=train_scenes,
        validation_scenes=val_scenes)
    chip_options = ObjectDetectionChipOptions(neg_ratio=1.0, ioa_thresh=0.8)
    predict_options = ObjectDetectionPredictOptions(
        merge_thresh=0.1, score_thresh=0.5)

    backend = PyTorchObjectDetectionConfig(
        model=ObjectDetectionModelConfig(backbone=Backbone.resnet50),
        solver=SolverConfig(
            lr=1e-4,
            num_epochs=10,
            test_num_epochs=2,
            batch_sz=16,
            one_cycle=True),
        log_tensorboard=True,
        run_tensorboard=False,
        test_mode=test)

    return ObjectDetectionConfig(
        root_uri=root_uri,
        dataset=dataset,
        backend=backend,
        train_chip_sz=chip_sz,
        predict_chip_sz=chip_sz,
        chip_options=chip_options,
        predict_options=predict_options)
